/*
 * eXist-db Open Source Native XML Database
 * Copyright (C) 2001 The eXist-db Authors
 *
 * info@exist-db.org
 * http://www.exist-db.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.exist.xquery.functions.util;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.exist.dom.persistent.NodeSet;
import org.exist.dom.QName;
import org.exist.xquery.AnalyzeContextInfo;
import org.exist.xquery.BasicFunction;
import org.exist.xquery.Cardinality;
import org.exist.xquery.Dependency;
import org.exist.xquery.FunctionSignature;
import org.exist.xquery.Profiler;
import org.exist.xquery.XPathException;
import org.exist.xquery.XQueryContext;
import org.exist.xquery.value.FunctionParameterSequenceType;
import org.exist.xquery.value.FunctionReturnSequenceType;
import org.exist.xquery.value.Sequence;
import org.exist.xquery.value.SequenceType;
import org.exist.xquery.value.StringValue;
import org.exist.xquery.value.Type;

/**
 * @author wolf
 * 
 */
public class IndexType extends BasicFunction {
	
	protected static final Logger logger = LogManager.getLogger(IndexType.class);

    public final static FunctionSignature signature = new FunctionSignature(
            new QName("index-type", UtilModule.NAMESPACE_URI, UtilModule.PREFIX),
                "Returns the range index type for a set of nodes or an empty sequence if no index is defined. ", 
                new SequenceType[] {
                    new FunctionParameterSequenceType("set-of-nodes", Type.NODE, Cardinality.ZERO_OR_MORE, "The set of nodes")
                    },
            new FunctionReturnSequenceType(Type.STRING, Cardinality.ZERO_OR_ONE, "the range index type"));

    public IndexType(XQueryContext context) {
        super(context, signature);
    }

    public void analyze(AnalyzeContextInfo contextInfo) throws XPathException {
        contextInfo.addFlag(NEED_INDEX_INFO);
        super.analyze(contextInfo);
        contextInfo.removeFlag(NEED_INDEX_INFO);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.exist.xquery.BasicFunction#eval(org.exist.xquery.value.Sequence[],
     *      org.exist.xquery.value.Sequence)
     */
    public Sequence eval(Sequence[] args, Sequence contextSequence) throws XPathException {
    	
        if (context.getProfiler().isEnabled()) {
            context.getProfiler().start(this);       
            context.getProfiler().message(this, Profiler.DEPENDENCIES, "DEPENDENCIES", Dependency.getDependenciesName(this.getDependencies()));
            if (contextSequence != null)
                {context.getProfiler().message(this, Profiler.START_SEQUENCES, "CONTEXT SEQUENCE", contextSequence);}
         }  

        Sequence result;

    	if (args[0].isEmpty())
            {result = Sequence.EMPTY_SEQUENCE;}
    	else {
	        final NodeSet nodes = args[0].toNodeSet();
	        //Remember it is the default value when no index is defined
	        if (nodes.getIndexType() == Type.ANY_TYPE)
	        	{result = Sequence.EMPTY_SEQUENCE;}
	        else
	        	{result = new StringValue(Type.getTypeName(nodes.getIndexType()));}  
    	}
    	//TODO : consider modularized indexes. We should thus return a * sequence...
    	
        if (context.getProfiler().isEnabled()) 
            {context.getProfiler().end(this, "", result);} 

        return result;
    }
}
